/* Program  :	RTFLTSEQ.C
   Purpose  :	This program was used to prototype the new algorythm for
					determining filter sequences in the REF TEK DAS and can
					be used to demonstrate the algorythm.
   Host     :	NEC 386/20   MS-DOS 3.30   Borland C v2.0
   Target   :	PC compatibles
   Packages :	None.
   Author   :	Phillip L. Davidson
   Company  :	Refraction Technology, Inc.
					2626 Lombardy Lane, Suite 105
					Dallas, Texas  75220
					(214) 353-0609     Fax (214) 353-9659
   Copyright:	(c) 1992 - 1993 Refraction Technology, Inc.
   Notes    :	The functions snsfnc() and get_seq() are identical to the
					function in the DAS.  wtch_dog(), set_ad_params(), and
					set_atd_params() are dummy functions.  set_dsp_params() prints
					out the strings.

					The user must enter the sample rate and included channels
					for each data stream.  In the DAS, these are set by sending
					and implementing parameters.  Either way, the sample rates
					are stored by stream in b_smp_rate[], the number of channels
					are stored by stream in b_num_chn[], and the state of each
					channel for each stream is stored in b_chn_num[][].  For
					the channels, 0 means the channel is excluded.
	
					snsfnc() determines the string and stage settings for the
					filter strings passed to the DSP.  Each string has a single
					channel input with multiple stages.  Each stage consists of
					a filter number, a decimation factor, and several possible
					streams for outputs.  Stage 0 is special, in that its filter
					number is the input channel number and its decimation factor
					is always -1.  Unused strings, stages or outputs always
					contain -1.

					Strings are built by starting with the largest sample rate
					requested and continuing to filter the output to as many
					other requested rates as possible.  A 'master string' is
					built using only the data stream numbers.  Once the master
					string is complete, it is duplicated for each channel used
					by each stream which was included in the string.  All channels
					for a stream will go through the same sequence of filters.
					
					An array of channel indicators is maintained for each stage in
					order to ensure the proper outputs when the master string is
					duplicated.  The high bit of each indicator ensures the use
					of the filter stage for the channel.  The rest of the bits
					indicate which of the multiple outputs needs this channel.
					Each bit position corresponds to one of the output stream
					numbers.

					The exact sequence of filters between any two rates is
					determined by the function get_seq().  It is recursive and
					tries all available combinations of decimations listed in the
					static array dec_factor[].  get_seq() has not changed since its
					original development by Michael B. Early in 1987.  When a
					sequence has been found, and there are sufficient stages still
					available, the sequence is copied into the master string and
					the channel indicators updated.

					snsfnc() returns an error code which is also returned by this
					program to the OS:
							0	no error occured
							1	at least one of the sample rates was skipped, due to
									an invalid rate, insufficient filter stages, or
									insufficient filter strings.
							2	all of the sample rates were invalid and skipped

					All user prompts are written to stderr.  This allows the
					output of the program to be redirected to a file or a printer
					but still see the prompts on the terminal screen.
		
	Revised:
*/
#define	VERSION	"1.1"
/*
		11May93	PLD - bug fix in snsfnc(),
					PLD - modify debug output
					PLD - add more error checking
#define	VERSION	"1.0"
		23Dec92	PLD - initial development
*/

#include	<stdio.h>							/* for prototype prg only */
#define	PROGRAM	"RTFLTSEQ"

/* code conventions - normally handled by convent.h */
/*#include	"convent.h"*/
#define	uchar		unsigned char
#define	ushort	unsigned short
#define	FORMFEED	(uchar)(12)
#define	OR			||

/* max values - normally handled by cfg_max.h */
/*#include	"cfg_max.h"*/
#define	max_chn		6						/* channels */
#define	max_stm		8						/* data streams */
#define	max_stg		9						/* filter stages in a string */
#define	max_strng	16						/* filter strings for DSP */

/* hardware description values - normally handled by cfg_sys.h */
/*#include	"cfg_sys.h"*/
struct	{
	/* first byte is config bits set by software */
			int				: 4;				/* filler */
			int	rt327		: 1;				/* 0 = no rt327, 1 = rt327 installed */
			int	rt299		: 1;				/* 0 = no rt299, 1 = rt299 installed */
			int	rt303		: 1;				/* 0 = no rt303, 1 = rt303 installed */
			int	rt305		: 1;				/* 0 = rt292, 1 = rt305 comm card */

	/* second byte is config bits from backplane unit id */
			int				: 2;				/* filler */
			int	fastclk	: 3;				/* 0 = 3.6864M, 1 = x1.5, 2 = x2.0, ... */
			int	db			: 1;				/* 0 = counts, 1 = dB filter card */
			int	spwr		: 1;				/* rt72:  0 = testbus, 1 = separate SCSI relay */
													/* rt72a: 0 = rt72a,   1 = rt44d */
			int	rt72a		: 1;				/* 0 = rt72, 1 = rt72a */
			}	Hd_cfg;							/* hardware configuration */

#define	DBGPRT	2							/* 0 = no output, 4 = all output */

#define	MAX_SMP_RATE	1000				/* raw data rate */

/* filter set definitions   */
#define	F2A		0
#define	F4A		1
#define	F5A		2
#define	F2			3
#define	F4			4
#define	F5			5

/* the number of decimation factors   */
#define	NUM_DECS		3

/* static tables */
short	dec_factor[NUM_DECS] = { 5, 4, 2 };
short	easy_filt[NUM_DECS] = { F5A, F4A, F2A };
short hard_filt[NUM_DECS] = { F5, F4, F2 };

/* function prototypes */
short		snsfnc();
short		get_seq( short, short, short [][2], short, short );

/* global variables */
ushort	b_usr_rate[max_stm];
ushort	b_smp_rate[max_stm];
ushort	b_num_chn[max_stm];
ushort	b_chn_num[max_stm][max_chn];

/********************************  main()  *********************************/
main()
	{
	ushort	strm,								/* stream counter */
				chan;								/* channel counter */
	short		result;
	uchar		chan_list[max_chn+2],		/* for user input */
				ch;

	printf("\n\n%s v%s  (c) 1992 - 1993 Refraction Technology, Inc.\n", PROGRAM, VERSION);

/* init Hd_cfg */
	Hd_cfg.rt327 = 0;
	Hd_cfg.rt299 = 0;
	Hd_cfg.rt303 = 0;
	Hd_cfg.rt305 = 1;

	Hd_cfg.fastclk = 0;
	Hd_cfg.db = 0;
	Hd_cfg.spwr = 0;
	Hd_cfg.rt72a = 1;

/* get user input */
	for (strm = 0; strm < max_stm; strm++)
		{
		for (chan = 0; chan < max_chn; chan++)
			b_chn_num[strm][chan] = 0;
		b_num_chn[strm] = 0;

		fprintf(stderr,"\nStream %2d: Sample rate? (0 to skip) ", strm+1);
		scanf("%d", &b_smp_rate[strm]);
		if (b_smp_rate[strm] != 0)
			{
			fprintf(stderr,  "                             Chans? ");
			scanf("%6s", chan_list);
			for (chan = 0; chan < max_chn; chan++)
				{
				ch = chan_list[chan];
				if (ch <= '0' || ch > (max_chn+0x30))
					break;
				b_chn_num[strm][ch-0x31] = 1;
				b_num_chn[strm]++;
				}
			}
		}

	for (strm = 0; strm < max_stm; strm++)
		{
		printf("\nStream %2d: %4d SPS on %d Channels ",
								strm, b_smp_rate[strm], b_num_chn[strm]);
		for (chan = 0; chan < max_chn; chan++)
			if (b_chn_num[strm][chan] != 0)
				printf(" %1d", chan);
			else
				printf("  ");
		}

/* process input & print result */
	result = snsfnc();
	printf("\n\nReturn: %d", result);
	return result;
	}
/****************************  end of main()  ******************************/

/***************************  dummy functions  *****************************/
wtch_dog()
	{
	return;
	}

void	set_ad_params()
	{
	return;
	}

void	set_atd_params()
	{
	return;
	}
/************************  end of dummy functions  *************************/

/***************************  set_dsp_params()  ****************************/
short		set_dsp_params( sns_def )
	/*signed*/short	sns_def[max_strng][max_stg][5];
	{
	ushort	string, stage, value;

	printf("\n\nSend strings to DSP:");
	for (string = 0; string < max_strng; string++)
		{
		printf("\n\nString %2d (Ch %1d):", string, sns_def[string][0][0]);
		for (stage = 0; stage < max_stg; stage++)
			{
			printf("\nStage %d:", stage);
			for (value = 0; value < 5; value++)
				printf(" %2d", sns_def[string][stage][value]);
			}
		}
	return 0;
	}
/************************  end of set_dsp_params()  ************************/

/******************************  get_seq()  ********************************/
short	get_seq( cur_rate, tgt_rate, sequence, level, max_dec )
	short	cur_rate,
			tgt_rate,
			sequence[][2],
			level,
			max_dec;
/* Purpose:	this module is recursive, and is used to determine the sequence
			of decimation values needed to get from one rate to another using
			the given rates in dec_factors[3] (this can be expanded later)
			neccessary parameters are:
				input rate, target rate, pointer to seq array, level num (0-8),
					and max dec value
   Returns:	0 for failure, 1 for success
				on failure, max dec value should be decreased and then try again
				on success, the seq is the array up to the first 0 entry
   Revised:
		10May93	PLD - change debug output
*/

{
short		i, j;

#if DBGPRT >= 4 /* lo pri prt */
printf("\n--get_seq: lev %d, cur %d, tgt %d, max_dec %d", level, cur_rate, tgt_rate,
																	max_dec);
#endif

wtch_dog();								/* kick the dog, this process can be 
												time consumming */
for ( i=0; i<NUM_DECS; i++)		/* check for each decimation factor  */
	{
	if ( (cur_rate % dec_factor[i] == 0)  &&  (max_dec >= dec_factor[i]) )
		{
		if ( (cur_rate / dec_factor[i]) == tgt_rate )
			{
#if DBGPRT >= 4 /* lo pri prt */
printf(", use_dec %d", dec_factor[i]);
#endif
			sequence[level][0] = hard_filt[i];
			sequence[level][1] = dec_factor[i];
			return 1;
			}
		if ( (cur_rate / dec_factor[i]) > tgt_rate )
			{
#if DBGPRT >= 4 /* lo pri prt */
printf(", use_dec %d", dec_factor[i]);
#endif
			sequence[level][0] = easy_filt[i];
			sequence[level][1] = dec_factor[i];
			for ( j=0; j<NUM_DECS; j++)
				{
				if ( (level+1) >= max_stg )
					{							/* return failed if we exceed the
													allowed number of levels (stages) */
					return 0;
					}
				if ( get_seq( cur_rate/dec_factor[i], tgt_rate, sequence,
																level+1, dec_factor[j] ) )
					{
					return 1;
					}
				}   /*  end of 'for' loop thru sub dec factors  */
			}
		}
	}  /* end of 'for' loop thru current dec factors  */
sequence[level][0] = -1;			/* no valid filter here */
return 0;
}
/**************************  end of get_seq()  *****************************/

/*******************************  snsfnc()  ********************************/
short	snsfnc()
	{
	/*signed*/short	*temp_ptr;
	/*signed*/short	strings[max_strng][max_stg][5];
#define	I_SNS_FLT	0						/* [][][0]=input or filt */
#define	I_SNS_DEC	1						/*	[][][1]=decimation factor */
#define	I_SNS_OUT1	2						/*	[][][2]=output 1 stream # */
#define	I_SNS_OUT2	3						/*	[][][3]=output 2 stream # */
#define	I_SNS_OUT3	4						/*	[][][4]=output 3 stream # */
#define	MAX_SNS_OUT	2						/* max # outputs allowed per stage */

	uchar					chan_list[max_stg][max_chn]; /* chan list for string dupl */
#define	M_CHN_FLT	((uchar)(0x10))	/* mask indicating channel use of filter */

	ushort				rates[max_stm][2];
#define I_RT_SPS		0						/*	[][0] = sample rate */
#define I_RT_STM		1						/* [][1] = stream num */

	ushort				i,j,k,				/* general purpose counters */
							strm,					/* stream counter */
							chan,					/* channel counter */
							stage,				/* stage counter */
							master_string,		/* index for dupl master string */
							error_flag;			/* return value */
#define	ERR_NONE		0						/* no error - good return */
#define	ERR_INCMPL	1						/* incomplete - some stream(s) skipped */
#define	ERR_INVLD	2						/* all sample rates invalid */

	ushort				r_active,			/* number of active rates */
							r_index,				/* rates index */
							r_swap;				/* swap index */
	short					r_seq[max_stg][2]; /* array of filter sequences */
#define I_SEQ_FLT		0						/*	[][0] = filter number */
#define I_SEQ_DEC		1						/* [][1] = decimation */

	ushort				cur_string,			/* current string */
							cur_stage,			/* current string stage */
							cur_out,				/* current stage output */
							cur_rate;			/* current sample rate */

	ushort				new_rate,			/* new sample rate */
							new_stream;			/* actual stream of new sample rate */

	if ( Hd_cfg.rt299 || Hd_cfg.rt327 )	/* RT299 or RT327 (dumb A/D) installed (pld) */
		{
		set_atd_params();						/* set parameters */
		error_flag = ERR_NONE;
		}  /* end of RT299 code */
	else if ( Hd_cfg.rt303 )				/* RT303 (24-bit A/D) installed (pld) */
		{
		set_ad_params();						/* set 24-bit parameters */
		error_flag = ERR_NONE;
		}  /* end of RT303 code */
	else											/* DSP installed */
		{
/*=========================================================================*/
		/* init return value */
		error_flag = ERR_NONE;

		/* init rates */
		for (strm = 0; strm < max_stm; strm++)
			{
			rates[strm][I_RT_SPS] = 0;
			rates[strm][I_RT_STM] = 0;
			}

		/* init the strings array to -1, or empty */
		temp_ptr = (short *)strings;
		for (i = 0; i < (max_strng*max_stg*5); i++)
			*temp_ptr++ = -1;

/*=========================================================================*/
		/* use insertion sort to fill in rates */
		r_active = 0;
		for (strm = 0; strm < max_stm; strm++)
			{
			if (b_num_chn[strm] == 0)
				continue;						/* ..with for (strm...) */

			new_rate = b_smp_rate[strm];
			if ( (new_rate > MAX_SMP_RATE)
					OR (new_rate == 0)
						OR ((MAX_SMP_RATE % new_rate) != 0) )
				{
#if DBGPRT
printf("\n\nSkipping stream %d - invalid rate of %d SPS", strm, new_rate);
#endif
				error_flag = ERR_INCMPL;
				continue;						/* ..with for (strm...) */
				}

			r_index = 0;
			while (r_index < r_active)		/* find where to insert rate */
				{
				cur_rate = rates[r_index][I_RT_SPS];
				if (new_rate > cur_rate)	/* if faster rate */
					break;
				if (new_rate == cur_rate)	/* if equal rate with more channels */
					if (b_num_chn[strm] > b_num_chn[ rates[r_index][I_RT_STM] ])
						break;
				r_index++;
				}

			r_swap = r_active;
			while (r_swap > r_index)		/* move lower rates down one */
				{
				rates[r_swap][I_RT_STM] = rates[r_swap-1][I_RT_STM];
				rates[r_swap][I_RT_SPS] = rates[r_swap-1][I_RT_SPS];
				r_swap--;
				}

			rates[r_index][I_RT_STM] = strm;	/* insert the new rate */
			rates[r_index][I_RT_SPS] = new_rate;
			r_active++;
			}  /* end for (strm = 0; strm < max_stm; strm++) */
#if DBGPRT
printf("\n\n%d Active Rates:", r_active);
for (i = 0; i < max_stm; i++)
	printf(" %2d:%d", rates[i][I_RT_STM], rates[i][I_RT_SPS]);
#endif

/*=========================================================================*/
		/* now build strings - create master in 1st available & duplicate
			for needed chans */
		cur_string = 0;
		for ( strm = 0 ; strm < r_active; strm++ )/* loop thru sorted sample rates */
			{
			r_index = strm;
			cur_rate = MAX_SMP_RATE;		/* start with max sample rate .. */
			cur_stage = 0;						/* .. first filter stage .. */
			cur_out = 0;						/* .. first output .. */
			for (stage = 0; stage < max_stg; stage++)
				for (chan = 0; chan < max_chn; chan++)
					chan_list[stage][chan] = 0; /* .. and no channels */

#if DBGPRT >= 3
printf("\n\nr_index: %d  rates:", r_index);
for (i = 0; i < max_stm; i++)
	printf(" %2d:%d", rates[i][I_RT_STM], rates[i][I_RT_SPS]);
#endif
			while (r_index < r_active)
				{
				new_rate = rates[r_index][I_RT_SPS]; /* get new rate */
				if (new_rate == 0)			/* skip NULLs */
					{
					r_index++;
					continue;					/* .. with while (r_index..) */
					}
				new_stream = rates[r_index][I_RT_STM]; /* get actual stream */
				if (b_num_chn[new_stream] + cur_string > max_strng)
					{						/* too many channels for remaining strings */
#if DBGPRT
printf("\n\nSkipping stream %d - not enough strings", new_stream);
#endif
#if DBGPRT >= 3
printf("\nstrm %d, r_index %d, r_active %d, new_rate %d",
				strm,    r_index,    r_active,    new_rate);
#endif
					error_flag = ERR_INCMPL;
					rates[r_index++][I_RT_SPS] = 0;
					continue;					/* .. with while (r_index..) */
					}

				for (stage = 0; stage < max_stg; stage++)
					r_seq[stage][I_SEQ_DEC] = 0; /* clear sequence array */

				if (new_rate == cur_rate)
					{
					if (cur_out < MAX_SNS_OUT)
						{							/* put stream # in output list */
						strings[cur_string][cur_stage][I_SNS_OUT1+cur_out] = new_stream;

										/* add chans to list for all current stages */
						for (stage = 0; stage <= cur_stage; stage++)
							for (chan = 0; chan < max_chn; chan++)
								if (b_chn_num[new_stream][chan])
									{
									chan_list[stage][chan] |= M_CHN_FLT;
									if (stage == cur_stage)
										chan_list[stage][chan] |= (uchar)(1<<cur_out);
									}
											
						cur_out++;
						rates[r_index][I_RT_SPS] = 0; /* mark rate as used */
						}
					}
				else if ( get_seq( cur_rate, new_rate, r_seq, 0, dec_factor[0] ) )
					{								/* valid sequence: copy it into strings[][][] */
					for (stage = 0; stage < max_stg; stage++)
						{
						if (r_seq[stage][I_SEQ_DEC] == 0) /* done */
							break;

						if (cur_stage+stage+1 > max_stg) /* won't fit */
							{
							if (cur_stage+1 < max_stg)
								{
								strings[cur_string][cur_stage+1][I_SNS_FLT] = -1;
								strings[cur_string][cur_stage+1][I_SNS_DEC] = -1;
								}
							stage = 0;
							break;
							}

						strings[cur_string][cur_stage+stage+1][I_SNS_FLT] = r_seq[stage][I_SEQ_FLT];
						strings[cur_string][cur_stage+stage+1][I_SNS_DEC] = r_seq[stage][I_SEQ_DEC];
						}  /* end for (stage = 0; stage < max_stg; stage++) */

					if (stage != 0)
						{
						cur_stage += stage;	/* update current stage */

													/* put stream # in output list */
						strings[cur_string][cur_stage][I_SNS_OUT1] = new_stream;
													/* add chans to list for all current stages */
						for (stage = 0; stage <= cur_stage; stage++)
							for (chan = 0; chan < max_chn; chan++)
								if (b_chn_num[new_stream][chan])
									{
									chan_list[stage][chan] |= M_CHN_FLT;
									if (stage == cur_stage)
										chan_list[stage][chan] |= (uchar)(1);
									}

						rates[r_index][I_RT_SPS] = 0; /* mark rate as used */
						cur_rate = new_rate;	/* update current rate */
						cur_out = 1;			/* just filled in 1st output */
						}
					}
				else if (r_index == strm)
					{	/* sequence not found for starting rate */
#if DBGPRT
printf("\n\nSkipping stream %d - rate unreachable", new_stream);
#endif
#if DBGPRT >= 3
printf("\nstrm %d, r_index %d, r_active %d, new_rate %d",
				strm,    r_index,    r_active,    new_rate);
#endif
					error_flag = ERR_INCMPL;
					}

				r_index++;
				}  /* end while (r_index < r_active) */

			if (cur_string >= max_strng)
				continue;						/* .. with for (strm...) */

#if DBGPRT >= 2
													/* display the master string */
printf("\n\nMaster String (%2d): Ch:", cur_string);
for (chan = 0; chan < max_chn; chan++)
	printf("  %1d", chan);
for (stage = 0; stage < max_stg; stage++)
	{
	printf("\nStage %d:", stage);
	for (i = 0; i < 5; i++)
		printf(" %2d", strings[cur_string][stage][i]);
	for (chan = 0; chan < max_chn; chan++)
		printf(" %02x", chan_list[stage][chan]);
	}
#endif

													/* duplicate the master string */
			master_string = cur_string;
			for (chan = 0; chan < max_chn; chan++)
				{
				if (chan_list[0][chan])
					{
					if (master_string != cur_string) /* don't copy onto itself */
						{
						stage = 0;
						while ( chan_list[stage][chan] && stage < max_stg )
							{						/* only copy needed stages */
							strings[cur_string][stage][I_SNS_FLT] = strings[master_string][stage][I_SNS_FLT];
							strings[cur_string][stage][I_SNS_DEC] = strings[master_string][stage][I_SNS_DEC];
							j = 0;
							for (i = 0; i < MAX_SNS_OUT; i++) /* copy only needed outputs */
								{
								if ( chan_list[stage][chan] & (1<<i) )
									strings[cur_string][stage][I_SNS_OUT1+(j++)] =
													strings[master_string][stage][I_SNS_OUT1+i];
								}
							stage++;
							}
						}
					strings[cur_string++][0][I_SNS_FLT] = chan; /* input chan for string */
					}  /* end if (chan_list[0][chan]) */
				}  /* end for (chan = 0; chan < max_chn; chan++) */

													/* clean up 1st (old master) string */
			chan = strings[master_string][0][0];
			if (chan == 0xFFFF)				/* empty master string (pld) */
				{
#if DBGPRT >= 4
printf("\nBREAK");
#endif
				break;
				}

			for (stage = 0; stage < max_stg; stage++)
				{
				if (chan_list[stage][chan] == 0)	/* terminate the string */
					for (i = 0; i < 5; i++)
						strings[master_string][stage][i] = -1;
				else
					{
					k = 0;
					for (i = 0; i < MAX_SNS_OUT; i++)
						{
						if (strings[master_string][stage][I_SNS_OUT1+k] == -1)
							break;				/* quit on first inactive output */

						if ( !(chan_list[stage][chan] & (1<<i)) )
							{						/* don't need this output */
							for (j = k; j < MAX_SNS_OUT-1; j++)
								strings[master_string][stage][I_SNS_OUT1+j] =
											strings[master_string][stage][I_SNS_OUT1+1+j];
							strings[master_string][stage][I_SNS_OUT1+MAX_SNS_OUT-1] = -1;
							}
						else
							k++;
						}
					}
				}


#if DBGPRT >= 3
													/* display the copied strings */
printf("\n\nDuplicated Strings:");
while (master_string < cur_string)
	{
	printf("\nString %2d (Ch %1d):", master_string, strings[master_string][0][0]);
	for (stage = 0; stage < max_stg; stage++)
		{
		printf("\nStage %d:", stage);
		for (i = 0; i < 5; i++)
			printf(" %2d", strings[master_string][stage][i]);
		}
	master_string++;
	}
#endif

			}  /* end for ( strm < max_stm; strm++ ) */

		if (cur_string == 0)
			{
#if DBGPRT
printf("\n\nAll rates invalid");
#endif
			error_flag = ERR_INVLD;
			}

		set_dsp_params( strings );			/* now send the strings to the DSP */
		}

	return error_flag;
	}
/***************************  end of snsfnc()  *****************************/

