/* Program  :	RTFLTSEQ.C
   Purpose  :	This program incorporates the old algorythm for
					determining filter sequences in the REF TEK DAS and can
					be used to demonstrate the algorythm.
   Host     :	NEC 386/20   MS-DOS 3.30   Borland C v2.0
   Target   :	PC compatibles
   Packages :	None.
   Author   :	Phillip L. Davidson
   Company  :	Refraction Technology, Inc.
					2626 Lombardy Lane, Suite 105
					Dallas, Texas  75220
					(214) 353-0609     Fax (214) 353-9659
   Copyright:	(c) 1992 Refraction Technology, Inc.
   Notes    :	The functions find_hi() and get_seq() are identical to the
					functions in the DAS.  wtch_dog(), set_ad_params(), and
					set_atd_params() are dummy functions.  set_dsp_params()
					prints out the strings.  snsfnc() is identical except for
					minor modifications for output of the rates.

					The user must enter the sample rate and included channels
					for each data stream.  In the DAS, these are set by sending
					and implementing parameters.  Either way, the sample rates
					are stored by stream in b_smp_rate[], the number of channels
					are stored by stream in b_num_chn[], and the state of each
					channel for each stream is stored in b_chn_num[][].  For
					the channels, 0 means the channel is excluded.
	
					snsfnc() determines the string and stage settings for the
					filter strings passed to the DSP.  Each string has a single
					channel input with multiple stages.  Each stage consists of
					a filter number, a decimation factor, and several possible
					streams for outputs.  Stage 0 is special, in that its filter
					number is the input channel number and its decimation factor
					is always -1.  Unused strings, stages or outputs always
					contain -1.

					Strings are built by channel, starting with the largest sample
					rate requested across the streams for that channel and
					continuing to filter the output to as many of its requested
					rates as possible.

					The exact sequence of filters between any two rates is
					determined by the function get_seq().  It is recursive and
					tries all available combinations of decimations listed in the
					static array dec_factor[].  get_seq() has not changed since its
					original development by Michael B. Early in 1987.  When a
					sequence has been found, and there are sufficient stages still
					available, the sequence is copied into the master string and
					the channel indicators updated.

					This version of snsfnc() returns an error code which is also
					returned by this program to the OS:
							0	no error occured
							1	some error occured

					All user prompts are written to stderr.  This allows the
					output of the program to be redirected to a file or a printer
					but still see the prompts on the terminal screen.
		
	Revised:
*/
#define	VERSION	"1.0"
/*
		23Dec92	PLD - initial development
*/

#include	<stdio.h>							/* for prototype prg only */
#define	PROGRAM	"RTOLDSEQ"

/* code conventions - normally handled by convent.h */
/*#include	"convent.h"*/
#define	uchar		unsigned char
#define	ushort	unsigned short
#define	FORMFEED	(uchar)(12)
#define	FALSE		(0)
#define	TRUE		(1)

/* max values - normally handled by cfg_max.h */
/*#include	"cfg_max.h"*/
#define	max_chn		6						/* channels */
#define	max_stm		8						/* data streams */
#define	max_stg		9						/* filter stages in a string */
#define	max_strng	16						/* filter strings for DSP */

/* hardware description values - normally handled by cfg_sys.h */
/*#include	"cfg_sys.h"*/
struct	{
	/* first byte is config bits set by software */
			int				: 4;				/* filler */
			int	rt327		: 1;				/* 0 = no rt327, 1 = rt327 installed */
			int	rt299		: 1;				/* 0 = no rt299, 1 = rt299 installed */
			int	rt303		: 1;				/* 0 = no rt303, 1 = rt303 installed */
			int	rt305		: 1;				/* 0 = rt292, 1 = rt305 comm card */

	/* second byte is config bits from backplane unit id */
			int				: 2;				/* filler */
			int	fastclk	: 3;				/* 0 = 3.6864M, 1 = x1.5, 2 = x2.0, ... */
			int	db			: 1;				/* 0 = counts, 1 = dB filter card */
			int	spwr		: 1;				/* rt72:  0 = testbus, 1 = separate SCSI relay */
													/* rt72a: 0 = rt72a,   1 = rt44d */
			int	rt72a		: 1;				/* 0 = rt72, 1 = rt72a */
			}	Hd_cfg;							/* hardware configuration */

#define	DBGPRT	2							/* 0 = no output, 3 = all output */

#define	MAX_SMP_RATE	1000				/* raw data rate */

/* filter set definitions   */
#define	F2A		0
#define	F4A		1
#define	F5A		2
#define	F2			3
#define	F4			4
#define	F5			5

/* the number of decimation factors   */
#define	NUM_DECS		3

/* static tables */
short	dec_factor[NUM_DECS] = { 5, 4, 2 };
short	easy_filt[NUM_DECS] = { F5A, F4A, F2A };
short hard_filt[NUM_DECS] = { F5, F4, F2 };

/* function prototypes */
short		snsfnc();
short		get_seq( short, short, short [][2], short, short );

/* global variables */
ushort	b_usr_rate[max_stm];
ushort	b_smp_rate[max_stm];
ushort	b_num_chn[max_stm];
ushort	b_chn_num[max_stm][max_chn];

/********************************  main()  *********************************/
main()
	{
	ushort	strm,								/* stream counter */
				chan;								/* channel counter */
	short		result;
	uchar		chan_list[max_chn+2],		/* for user input */
				ch;

	printf("\n\n%s v%s  (c) 1992 Refraction Technology, Inc.\n", PROGRAM, VERSION);

/* init Hd_cfg */
	Hd_cfg.rt327 = 0;
	Hd_cfg.rt299 = 0;
	Hd_cfg.rt303 = 0;
	Hd_cfg.rt305 = 1;

	Hd_cfg.fastclk = 0;
	Hd_cfg.db = 0;
	Hd_cfg.spwr = 0;
	Hd_cfg.rt72a = 1;

/* get user input */
	for (strm = 0; strm < max_stm; strm++)
		{
		for (chan = 0; chan < max_chn; chan++)
			b_chn_num[strm][chan] = 0;
		b_num_chn[strm] = 0;

		fprintf(stderr,"\nStream %2d: Sample rate? (0 to skip) ", strm+1);
		scanf("%d", &b_smp_rate[strm]);
		if (b_smp_rate[strm] != 0)
			{
			fprintf(stderr,  "                             Chans? ");
			scanf("%6s", chan_list);
			for (chan = 0; chan < max_chn; chan++)
				{
				ch = chan_list[chan];
				if (ch <= '0' || ch > (max_chn+0x30))
					break;
				b_chn_num[strm][ch-0x31] = 1;
				b_num_chn[strm]++;
				}
			}
		}

	for (strm = 0; strm < max_stm; strm++)
		{
		printf("\nStream %2d: %4d SPS on %d Channels ",
								strm, b_smp_rate[strm], b_num_chn[strm]);
		for (chan = 0; chan < max_chn; chan++)
			if (b_chn_num[strm][chan] != 0)
				printf(" %1d", chan);
			else
				printf("  ");
		}

/* process input & print result */
	result = snsfnc();
	printf("\n\nReturn: %d", result);
	return result;
	}
/****************************  end of main()  ******************************/

/***************************  dummy functions  *****************************/
wtch_dog()
	{
	return;
	}

void	set_ad_params()
	{
	return;
	}

void	set_atd_params()
	{
	return;
	}
/************************  end of dummy functions  *************************/

/***************************  set_dsp_params()  ****************************/
short		set_dsp_params( sns_def )
	/*signed*/short	sns_def[max_strng][max_stg][5];
	{
	ushort	string, stage, value;

	printf("\n\nSend strings to DSP:");
	for (string = 0; string < max_strng; string++)
		{
		printf("\n\nString %2d (Ch %1d):", string, sns_def[string][0][0]);
		for (stage = 0; stage < max_stg; stage++)
			{
			printf("\nStage %d:", stage);
			for (value = 0; value < 5; value++)
				printf(" %2d", sns_def[string][stage][value]);
			}
		}
	return 0;
	}
/************************  end of set_dsp_params()  ************************/

/******************************  get_seq()  ********************************/
short	get_seq( cur_rate, tgt_rate, sequence, level, max_dec )
	short	cur_rate,
			tgt_rate,
			sequence[][2],
			level,
			max_dec;
/* Purpose:	this module is recursive, and is used to determine the sequence
			of decimation values needed to get from one rate to another using
			the given rates in dec_factors[3] (this can be expanded later)
			neccessary parameters are:
				input rate, target rate, pointer to seq array, level num (0-8),
					and max dec value
   Revised:	
   Returns:	0 for failure, 1 for success
				on failure, max dec value should be decreased and then try again
				on success, the seq is the array up to the first 0 entry
*/

{
short		i, j;

#if DBGPRT >= 3 /* lo pri prt */
printf("\n--get_seq,lev%d, cur%d, tgt%d, max%d", level, cur_rate, tgt_rate,
																	max_dec);
printf("\n dec at lev - 1 is %d --", sequence[level-1][1]);
#endif

wtch_dog();								/* kick the dog, this process can be 
												time consumming */
for ( i=0; i<NUM_DECS; i++)		/* check for each decimation factor  */
	{
	if ( (cur_rate % dec_factor[i] == 0)  &&  (max_dec >= dec_factor[i]) )
		{
		if ( (cur_rate / dec_factor[i]) == tgt_rate )
			{
			sequence[level][0] = hard_filt[i];
			sequence[level][1] = dec_factor[i];
			return 1;
			}
		if ( (cur_rate / dec_factor[i]) > tgt_rate )
			{
			sequence[level][0] = easy_filt[i];
			sequence[level][1] = dec_factor[i];
			for ( j=0; j<NUM_DECS; j++)
				{
				if ( (level+1) >= max_stg )
					{							/* return failed if we exceed the
													allowed number of levels (stages) */
					return 0;
					}
				if ( get_seq( cur_rate/dec_factor[i], tgt_rate, sequence,
																level+1, dec_factor[j] ) )
					{
					return 1;
					}
				}   /*  end of 'for' loop thru sub dec factors  */
			}
		}
	}  /* end of 'for' loop thru current dec factors  */
sequence[level][0] = -1;			/* no valid filter here */
return 0;
}
/**************************  end of get_seq()  *****************************/

/******************************  finc_hi()  ********************************/
short	find_hi( array )
	short array[max_stm][2];
/* Purpose:	this module will look thru an array of ints of form XXXX[16][2],
				and return the index to the highest int in XXXX[][0]
   Revised:	
   Returns:	the index to the highest int in XXXX[][0]
*/

	{
	short i, max, index;

#if DBGPRT >= 3 /* low pri prt */
printf("\n--find_hi--");
#endif
	index = -1;
	max = 0;
	for ( i=0; i<max_stm; i++)
		{
		if ( array[i][0] > max )
			{
			max = array[i][0];
			index = i;
			}
		}
	return index;
	}
/**************************  end of finc_hi()  *****************************/

#define STRMS_PER_STAGE 2	/* the max num stream output per DSP stage */
/*******************************  snsfnc()  ********************************/
short	snsfnc()
/* Purpose:	
   Revised:	
   Returns:	
*/

	{
/*signed*/short	sns_def[max_strng][max_stg][5];
											/* [string_num][stage_num][0=input or filt,
																		1=decimation factor
																		2=output stream
																		3=output stream
																		4=output stream ] */
short	rates[max_stm][2];			/* rate value in [][0], stream num in [][1] */
short sav_rates[max_stm][2];
short	sequence[max_stg][2];
short	strm_num, string, stage, source, r_index, hi_index,
		old_rate, i, j, error_flag;
short		*temp_ptr;

	if ( Hd_cfg.rt299 || Hd_cfg.rt327 )	/* RT299 or RT327 (dumb A/D) installed (pld) */
		{
		set_atd_params();						/* set parameters */
		error_flag = FALSE;
		}  /* end of RT299 code */
	else if ( Hd_cfg.rt303 )				/* RT303 (24-bit A/D) installed (pld) */
		{
		set_ad_params();						/* set 24-bit parameters */
		error_flag = FALSE;
		}  /* end of RT303 code */
	else											/* DSP installed */
		{

		/*  init the sns_def array to -1, or empty */
		temp_ptr = sns_def;			/* set all sns params to -1 */
		for ( i=0; i<(max_strng*max_stg*5); i++)
			{
			*temp_ptr = -1;
			temp_ptr++;
			}

			/*  start by going thru each of the max_chn possible source chans,
					and doing a check of the stream params looking for that
					input chan #
			*/

		string = 0;											/* string num to be done */
		error_flag = FALSE;
		for ( source = 0; source < max_chn; source++)
			{
			for (i=0; i<max_stm; i++)				/* zero rates for this source */
				{
				rates[i][0] = 0;
				sav_rates[i][0] = 0;
				}
			stage = 0;										/* stage num to be done */
			r_index = 0;									/* zero index */
			for ( strm_num=0; strm_num<max_stm; strm_num++)
																/* go thru each stream */
				{
				if ( b_chn_num[strm_num][source] == 1 )
					{
					if ( b_usr_rate[strm_num] )
						{
							/*  handle user rates here  */
						rates[r_index][0] = b_smp_rate[strm_num];
						rates[r_index++][1] = strm_num;
						}
					else
						{
						rates[r_index][0] = b_smp_rate[strm_num];
						rates[r_index++][1] = strm_num;
						}
					}
				}
#if DBGPRT >= 2
printf("\n\nRates=%d,%d,%d,%d,%d,%d,%d,%d,",rates[0][0],rates[1][0],rates[2][0],
			rates[3][0],rates[4][0],rates[5][0],rates[6][0],rates[7][0]);
printf("%d,%d,%d,%d,%d,%d,%d,%d",rates[8][0],rates[9][0],rates[10][0],
			rates[11][0],rates[12][0],rates[13][0],rates[14][0],rates[15][0]);
#endif
						/* now that all the needed rates for this source are
						compiled, start setting up the sns params */
			while ( ((hi_index = find_hi( rates )) != -1) ||
													(find_hi( sav_rates ) != -1) )
						/* find_hi = index to hi value, or = -1 if all vals = 0 */
				{
				if ( hi_index == -1 )
					{
									  	/* copy sav_rates to rates, and zero out
											save_rates, then get a
											good hi_index     */
					for ( i=0; i<max_stm; i++)
						{
						for ( j=0; j<2; j++)
							{
							rates[i][j] = sav_rates[i][j];
							sav_rates[i][j] = 0;
							}
						}
					hi_index = find_hi( rates );
															/* start a new string */
					string >= max_strng ? error_flag = TRUE: string++;
					stage = 0;
					}
											/* is rate out of range */
				if ( rates[hi_index][0] > 1000 )
					{
					rates[hi_index][0] = 0;
					error_flag = TRUE;
					continue;
					}
				if ( stage == 0 )
					{
					old_rate = 1000;
					if ( rates[hi_index][0] == 1000 )
						{
								/*  set sns input and output  */
						sns_def[string][stage][0] = source;
							/* check for and set the three possible streams */
						for ( i=0; i<STRMS_PER_STAGE; i++ )
							{
							short hi;

							hi = find_hi(rates);
							if ( (hi != -1) && (rates[hi][0] == 1000) )
								{
								sns_def[string][stage][2+i] = rates[hi][1];
																	/* set a stream output */
								rates[hi][0] = 0;		/* remove this entry */
								}
							else
								{
								sns_def[string][stage][2+i] = -1;	/* set no output */
								}
							}	/* end of 'for' loop thru possible stream outputs */
						stage++;
						}
					else		/* rates != 1000 and stage = 0 */
						{
						sns_def[string][stage][0] = source;
						sns_def[string][stage][2] = -1;
						sns_def[string][stage][3] = -1;
						sns_def[string][stage][4] = -1;
						stage++;
						}
					}
					/*  if rate = 1000, then this pass is done,
						if rate != 1000, then stage 0 is done, but we must
									get the filtering down to = rate
					*/
				for ( i=0; i<9; i++)	 		/*  zero out the filter sequence  */
					{
					sequence[i][0] = -1;
					}
				if ( rates[hi_index][0] != 0 )
					{
				/* get_seq( cur_rate, tgt_rate, sequence, level, max_dec ) */
					if ( get_seq( old_rate, rates[hi_index][0], sequence, 0,
																				dec_factor[0]) )
						{
						for ( i=0; sequence[i][0]!=(-1); i++)
							{
							sns_def[string][stage][0] = sequence[i][0];
							sns_def[string][stage][1] = sequence[i][1];
							stage++;
							}
						old_rate = rates[hi_index][0];
							/* check for and set the three possible streams */
						for ( i=0; i<STRMS_PER_STAGE; i++ )
							{
							short hi;

							hi = find_hi(rates);
							if ( (hi != -1) && (rates[hi][0] == old_rate) )
								{
								sns_def[string][stage-1][2+i] = rates[hi][1];
																	/* set a stream output */
								rates[hi][0] = 0;		/* remove this entry */
								}
							else
								{
								sns_def[string][stage-1][2+i] = -1;	/* set no output */
								}
							}	/* end of 'for' loop thru possible stream outputs */
						}
					else		/* !get_seq() */
						{
						if ( (stage == 1) && (old_rate != rates[hi_index][0]) )
							{
									/* if stage == 1, then the target sample rate
										is unattainable with the current decimation
										factors, ....   error time	  ....		*/
							error_flag = TRUE;
							rates[hi_index][0] = 0;	/* remove invalid entry */
							sns_def[string][0][0] = -1;	/* remove input to string */
							string--;					/* redo this string */
							}
						else
							{
							sav_rates[hi_index][0] = rates[hi_index][0];
							sav_rates[hi_index][1] = rates[hi_index][1];
							rates[hi_index][0] = 0;		/* remove this entry */
							}
						}

					}

				}			/*  end of while( hi_index != -1 )  */
			if ( stage != 0 )
				{
				string++;
				}
			if ( string > max_strng )
				{
				error_flag = TRUE;		/* have made too many strings */
				string = max_strng - 1;
				}

			}		/* end of source 'for' loop  */

		/* here we may send the commands to set the dsp filters */

		set_dsp_params( sns_def );
		}  /* end of DSP code */


	if ( error_flag )
		{
		return 1;
		}
	else
		{
		return 0;
		}

	}
/***************************  end of snsfnc()  *****************************/

