#define MAIN
#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <math.h>
#include <ctype.h>
#include <string.h>
#include <malloc.h>
#include <memory.h>

/**
   NAME: interpol
   SYNOPSIS:

       interpol

   VERSION: 1.0
   DATE: 1996-05-15
   DESCRIPTION: Change sampling rate of an ASCII input file by interpolation.
   Uses E. Wielandt's interpolation operators

**/
main(argc,argv)
        int argc;
        char    *argv[];

{
    long i,k;

    FILE *fo;
    FILE *fi;
    char in_name[80];
    char out_name[80];
    long nskip;
    int conv_fac;
    int no_it;     /* no of iterations */
    int next_pow2; /* next power of 2 */
    long zlen = 15; /* no. of zeros to be appended and prepended 
                      make it at least as large as the interpolation filter
                   */
    char ch;
    long ntot;
    long ndat,ndat2;
    double *tr;
    double x;
    char text_buffer[255];
    char *string_end;    /* pointer to possible CR */
    long offset;

    void intpol2();

    nskip = 0L;
    if (argc < 5)
    {
       printf("USAGE: interpol  -i <input file> -o <output file> -f <interpolation factor; power of 2 (2)> \n               [-s <no. pts to skip (0)>]\n");
       exit(1);
    }
    conv_fac = 2;
    for (i=1; i<argc;i++)
    {
        if (argv[i][0] == '-' && strlen(argv[i]) >= 2 )
        {
            switch( argv[i][1])
            {
                case 'i':  /* input file name */
                case 'I':  /* input file name */
                {
                    i++;
                    strcpy(in_name,&argv[i][0]);
                    break;
                }
                case 'o':  /* output file name */
                case 'O':  /* output file name */
                {
                    i++;
                    strcpy(out_name,&argv[i][0]);
                    break;
                }
                case 's':  /* number of lines to skip on input */
                case 'S':
                {
                    i++;
                    nskip = atol(&argv[i][0]);
                    break;
                }
                case 'f':   /* interpolation factor */
                case 'F':
                {
                    i++;
                    conv_fac = atoi(&argv[i][0]);
                    break;
                }
            }
        }
    }

    /* estimate no. of lines in file */
    ntot = 0L;
    if ((fo = fopen(in_name,"rt")) != NULL)
    {
            while((ch=fgetc(fo)) != EOF)
            {
                if (ch == '\n')
                    ntot++;
            }
        fclose(fo);
    }
    ndat = ntot - nskip;

    /* calculate next power of 2 */
    next_pow2 = 1;
    no_it = 0;
    while(next_pow2 < conv_fac)
    {
        next_pow2 *=2;
        no_it +=1;
    }
    if(next_pow2 != conv_fac)
        printf("Interpolate by factor %d (requested %d)\n",next_pow2,conv_fac);


    tr = (double *)calloc((ndat+2*zlen)*next_pow2 ,sizeof(double));

    /* INPUT  */
    if((fi = fopen(in_name,"rt")) == NULL)
    {
            printf("input file %s cannot be opened!\n",in_name);
            exit(1);
    }

    for(k=0;k<ndat+nskip;k++)
    {
        if(fgets(text_buffer,255,fi) != NULL)
        {
            if ((string_end = memchr(&text_buffer[0],'\n',255)) != NULL)
                    *string_end = '\0';
            sscanf(text_buffer,"%lf",&x);
        }
        else
            x = 0.0;
        if(k >= nskip)
            *(tr+k-nskip+zlen) = x;
    }
    fclose(fi);

    ndat2 = ndat + 2*zlen;
    offset = zlen;
    for(k=0;k<no_it;k++)
    {
       intpol2(tr-1,ndat2);
       offset *= 2;
       ndat2 *= 2;
       ndat *= 2;
    }

    /* OUTPUT  */
    if((fo = fopen(out_name,"wt")) == NULL)
    {
            printf("output file %s cannot be opened!\n",out_name);
            exit(1);
    }
  
    for(k=0;k<ndat;k++)
    {
        fprintf(fo,"%lf\n",*(tr+k+offset));

    }
    fclose(fo);
    free((char *)tr);

}

