	program fircorr
c
c ...... FIR correction filter
c
c	Corrects for non-causal effects of the FIR filters used
c	the Reftek 72A-08 dataloggers.  The current implimentation
c	only corrects 20 Hz and 50 Hz data streams recorded by the
c	SAPSE experiment.
c
c	Bob Uhrhammer - March 1997
c
	implicit none
	integer*4 yr, doy, hr, mn, sc, us, np, fcn
	common /fc_1/ yr, doy, hr, mn, sc, us, np, fcn
	character stn*4, str*3, net*2
	common /fc_2/ stn, str, net
	real*8 dt, tshift, x
	common /fc_3/ dt, tshift, x(180060)
	character*80 msifn, msofn
	common /fc_4/ msifn, msofn
	integer*4 lnb, lnblnk
c
c ...... input miniseed file name
c
	call fc_cli
	write(*,*) 'FIRCORR - Version 1.0 - 14 March 1997'
c
c ...... read miniseed file
c
	call fc_read
c
c ...... correct for non-causal FIR filter effects
c
	call fc_fir2caus
c
c ...... write FIR corrected miniseed file
c
	lnb=lnblnk(msifn)
	if(msifn(lnb-16:lnb-16).eq.'D') then
	  msofn=msifn(1:lnb-17)//'F'//msifn(lnb-15:lnb)
	elseif(msifn(lnb-14:lnb-14).eq.'D') then
	  msofn=msifn(1:lnb-15)//'F'//msifn(lnb-13:lnb)
	else
	  write(*,*) 'Invalid file name'
	  call exit(1)
	endif
c
	call fc_write
c
	stop
	end

	subroutine fc_fir2caus
c
c ...... apply fir correction filter
c	 adapted from Frank Scherbaum's fir2caus program
c
	implicit none
	integer*4 i, j, k, l
	integer*4 yr, doy, hr, mn, sc, us, np, fcn
	common /fc_1/ yr, doy, hr, mn, sc, us, np, fcn
	character stn*4, str*3, net*2
	common /fc_2/ stn, str, net
	real*8 dt, tshift, x
	common /fc_3/ dt, tshift, x(180060)
	character*80 msifn, msofn
	common /fc_4/ msifn, msofn
	real*8 ar20, ma20, ar50, ma50
	common /fc_5/ ar20(29), ma20(30), ar50(27), ma50(28)
	real*8 ttc20, ttc50, sum, avg
	integer*4 n30, np1, j, nar20, nma20, nar50, nma50
	real*8 xr(180060), y(180060)
c
	data n30 / 30 /
	data nar20,nma20,nar50,nma50 / 29,30,27,28 /
	data ttc20, ttc50 / 30.,28.5 /
c
	sum=0.d0
	do 1 i=1,np
	  sum=sum+x(i)
    1   continue
	avg=sum/dble(np)
	j=0
	do 2 i=1,n30
	  j=j+1
	  xr(j)=0.d0
	  xr(j+np)=0.d0
    2   continue
	do 3 i=np,1,-1
	  j=j+1
	  xr(j)=x(i)
    3   continue
	np1=j+30
c
	if(fcn.eq.1) then
	  tshift=ttc20*dt
	  do 6 i=1,np1
	    y(i)=0.d0
	    do 4 l=1,nma20
	      if((i-l).ge.1) y(i)=y(i)+xr(i-l)*ma20(l)
    4       continue
	    do 5 k=1,nar20
	      if((i-k).ge.1) y(i)=y(i)+y(i-k)*ar20(k)
    5       continue
    6     continue
	elseif(fcn.eq.2) then
	  tshift=ttc50*dt
	  do 9 i=1,np1
	    y(i)=0.d0
	    do 7 l=1,nma50
	      if((i-l).ge.1) y(i)=y(i)+xr(i-l)*ma50(l)
    7       continue
	    do 8 k=1,nar50
	      if((i-k).ge.1) y(i)=y(i)+y(i-k)*ar50(k)
    8       continue
    9     continue
	endif
c
	j=0
	do 10 i=np+n30,n30+1,-1
	  j=j+1
	  x(j)=y(i)
   10   continue
c
	return
	end

	block data
c
c ..... recursion coefficients
c
	implicit none
	integer*4 i
	real*8 ar20, ma20, ar50, ma50
	common /fc_5/ ar20(29), ma20(30), ar50(27), ma50(28)
c
	data (ar20(i),i=1,16) /
     1   1.737195704118395d-1 ,  1.101883218047552d-1 ,
     2  -2.227260570281201d-1 ,  2.467320865538899d-1 ,
     3  -2.210970905645717d-1 ,  1.720605392065184d-1 ,
     4  -1.157547744952389d-1 ,  6.319615499483124d-2 ,
     5  -2.077865836602879d-2 , -8.839173236533830d-3 ,
     6   2.581488052682035d-2 , -3.217926047585207d-2 ,
     7   3.091423042477604d-2 , -2.518304652784169d-2 ,
     8   1.773797649268340d-2 , -1.061932734463971d-2 /
	data (ar20(i),i=17,29) /
     1   4.985192027793689d-3 , -1.302938395730585d-3 ,
     2  -6.703602426439948d-4 ,  1.165326062170634d-3 ,
     3  -1.299242223375366d-3 ,  3.801316995198907d-4 ,
     4   1.298171649702394d-7 , -2.314750093580197d-6 ,
     5   1.695357183597308d-7 ,  2.295153994187671d-7 ,
     6  -8.230015144937490d-8 , -7.226061914641452d-12,
     7   2.807772251743828d-31 /
	data (ma20(i),i=1,16) /
     1  -2.807772251743828d-31, 7.226061914641452d-12 ,
     2   8.23001514493749d-8  , -2.295153994187671d-7 ,
     3  -1.695357183597308d-7 ,  2.314750093580197d-6 ,
     4  -1.298171649702394d-7 , -0.0003801316995198907,
     5   0.001299242223375366 , -0.001165326062170634 ,
     6   0.0006703602426439948,  0.001302938395730585 ,
     7  -0.004985192027793689 ,  0.01061932734463971  ,
     8  -0.0177379764926834   ,  0.02518304652784169  /
	data (ma20(i),i=17,30) /
     1  -0.03091423042477604  ,  0.03217926047585207  ,
     2  -0.02581488052682035  ,  0.00883917323653383  ,
     3   0.02077865836602879  , -0.06319615499483124  ,
     4   0.1157547744952389   , -0.1720605392065184   ,
     5   0.2210970905645717   , -0.2467320865538899   ,
     6   0.2227260570281201   , -0.1101883218047552   ,
     7  -0.1737195704118395   ,  1.d0 /
	data (ar50(i),i=1,16) /
     1  -0.1242048932575708   ,  0.2662598807272334   ,
     2  -0.26650387060014     ,  0.2184232793460652   ,
     3  -0.1543949095570169   ,  0.0910447589430343   ,
     4  -0.03734260348941831  , -0.002550142986892878 ,
     5   0.02780436587686921  , -0.03990967221655575  ,
     6   0.04168448606662813  , -0.03658652084749543  ,
     7   0.02786588532067796  , -0.01835819095064078  ,
     8   0.00985257668948066  , -0.003557942481924384 /
	data (ar50(i),i=17,27) /
     1  -0.0004914937892993993,  0.00224769834221652  ,
     2  -0.002742214357876802 ,  0.002022262861171042 ,
     3  -0.001504860205621665 ,  0.0004385542213098363,
     4   0.0001731438385110338,  2.677193599444333d-6 ,
     5  -3.790778556495849d-7 ,  2.240541076767229d-10,
     6  -3.255726664368189d-27 /
	data (ma50(i),i=1,16) /
     1   3.255726664368189d-27, -2.240541076767229d-10,
     2   3.790778556495849d-7 , -2.677193599444333d-6 ,
     3  -0.0001731438385110338, -0.0004385542213098363,
     4   0.001504860205621665 , -0.002022262861171042 ,
     5   0.002742214357876802 , -0.00224769834221652  ,
     6   0.0004914937892993993,  0.003557942481924384 ,
     7  -0.00985257668948066  ,  0.01835819095064078  ,
     8  -0.02786588532067796  ,  0.03658652084749543  /
	data (ma50(i),i=17,28) /
     1  -0.04168448606662813  ,  0.03990967221655575  ,
     2  -0.02780436587686921  ,  0.002550142986892878 ,
     3   0.03734260348941831  , -0.0910447589430343   ,
     4   0.1543949095570169   , -0.2184232793460652   ,
     5   0.26650387060014     , -0.2662598807272334   ,
     6   0.1242048932575708   ,  1.d0 /
c
	end

	subroutine fc_cli
c
c ...... read miniseed input file name
c
	implicit none
	integer*4 lnb, lnblnk, narg, iargc
	character*80 msifn, msofn
	common /fc_4/ msifn, msofn
c
	narg=iargc()
	if(narg.ne.1) then
	  write(*,'(a)')     'RF2C - Reftek FIR to Causal Filter'
	  write(*,'(a)')     'Version 1.0 - 14 Mar 1997'
	  write(*,'(7x,a)')  'Usage:'
	  write(*,'(14x,a)') 'rf2c ms_file'
	  write(*,'(7x,a)')  'Where:'
	  write(*,'(14x,a)') 'ms_file - miniseed data file'
	  write(*,'(7x,a)')  'NOTE:'
	  write(*,'(14x,a)') 'This version of rf2c only processes'
	  write(*,'(14x,a)') '20 Hz & 50 Hz data recorded by Reftek'
	  write(*,'(14x,a)') '72A-08 recorders.  rf2c outputs a'
	  write(*,'(14x,a)') 'causal ms_file with a ".F." substituted'
	  write(*,'(14x,a)') 'for the ".D" in the file name.'
	  call exit(0)
	else
	  call getarg(narg,msifn)
	  lnb=lnblnk(msifn)
	  if((lnb.eq.2).and.(msifn(1:2).eq.'-h')) then
	    write(*,'(a)')     'RF2C - Reftek FIR to Causal Filter'
	    write(*,'(a)')     'Version 1.0 - 14 Mar 1997'
	    write(*,'(7x,a)')  'Usage:'
	    write(*,'(14x,a)') 'rf2c ms_file'
	    write(*,'(7x,a)')  'Where:'
	    write(*,'(14x,a)') 'ms_file - miniseed data file'
	    write(*,'(7x,a)')  'NOTE:'
	    write(*,'(14x,a)') 'This version of rf2c only processes'
	    write(*,'(14x,a)') '20 Hz & 50 Hz data recorded by Reftek'
	    write(*,'(14x,a)') '72A-08 recorders.  rf2c outputs a'
	    write(*,'(14x,a)') 'causal ms_file with a ".F." substituted'
	    write(*,'(14x,a)') 'for the ".D" in the file name.'
	    call exit(0)
	  endif
	endif
	open(10,file=msifn(1:lnb),status='old',err=1)
	return
c
    1   write(*,*) msifn(1:lnb),' is not a valid file'
	call exit(1)
	stop
	end

	subroutine fc_read
c
c ...... input miniseed file data
c
	implicit none
	integer*4 yr, doy, hr, mn, sc, us, np, fcn
	common /fc_1/ yr, doy, hr, mn, sc, us, np, fcn
	character stn*4, str*3, net*2
	common /fc_2/ stn, str, net
	real*8 dt, tshift, x
	common /fc_3/ dt, tshift, x(180060)
	character*80 msifn, msofn
	common /fc_4/ msifn, msofn
c
	call msread(msifn,stn,net,str,yr,doy,hr,mn,sc,us,dt,np,x)
	if(dt.eq.0.05d0) then
	  fcn=1
	elseif(dt.eq.0.02d0) then
	  fcn=2
	else
	  write(*,*) 'dt = ',dt
	  write(*,*) 'Invalid sample rate'
	  call exit(1)
	endif
c
	return
	end

	subroutine fc_write
c
c ...... write miniseed data file
c
	implicit none
	integer*4 yr, doy, hr, mn, sc, us, np, fcn
	common /fc_1/ yr, doy, hr, mn, sc, us, np, fcn
	character stn*4, str*3, net*2
	common /fc_2/ stn, str, net
	real*8 dt, tshift, x
	common /fc_3/ dt, tshift, x(180060)
	character*80 msifn, msofn
	common /fc_4/ msifn, msofn
c
	np=np-32
	call mswrite(msofn,stn,net,str,yr,doy,hr,mn,sc,us,
     1    dt,tshift,np,x)
c
	return
	end

	subroutine msread(file,stn,net,str,yr,doy,hr,mn,sc,us,
     1  dt,np,x)
c
c  Sample fortran program to read and uncompress data from 
c  a single channel MiniSEED data file.
c
	implicit none
 	include "/usr/local/include/qlib2.inc"

	integer MAXPTS
	parameter (MAXPTS = 180060)
	double precision THRESHOLD
	parameter (THRESHOLD = .5)

	record /DATA_HDR/ hdr, hdr1
	record /INT_TIME/ endtime
	record /EXT_TIME/ et1, et2
	integer data(MAXPTS)
	integer fp, nreq, nread, n, date_fmt, i
	integer seconds, usecs, islew
	double precision slew, thresh
	character*80 s1, s2
c
c ...... declaration of parameters in subroutine call list
c
	character*4 stn
	character*3 str
	character*2 net
	integer*4 yr, doy, hr, mn, sc, us, np, lnb
	real*8 dt, x(*)
	character*80 file

c external functions and subroutines
	external getarg
	integer lnblnk
	external lnblnk 
c functions in most fortran libraries.
	double precision tslew
	external tslew
	external f_read_ms, f_copy_data_hdr, f_time_interval
	external f_int_to_ext, f_time_to_str, f_add_time
	external f_delete_blockette
c functions in library fio.
	integer ifopen, ifclose
	external ifopen, ifclose

	
c open input miniseed file.
	fp = ifopen (file, "r")
	if (fp .eq. 0) then
		write(*,*) "unable to open file", file
		call exit(1)
	endif

c set number of requested data points.
c NOTE -- buffer must be large enough to handle this many points.
	nreq = MAXPTS

c read first block, and establish station, channel and timing thresholds.
	date_fmt = 0
	slew = 0
	nread = 0
	n = f_read_ms (hdr, data(nread+1), nreq, fp)
	if (n .gt. 0) then
		call f_copy_data_hdr (hdr1, hdr)
		call f_time_interval (1, hdr.sample_rate, seconds, usecs)
		thresh = (seconds * USECS_PER_SEC + usecs) * THRESHOLD
	endif

c update number of points remaining and continue read more blocks.
c check to ensure the data stream is for the came station and channel,
c and check for time tears.
	do while ((n .gt. 0) .and. (abs(slew) .lt. thresh))
	    nread = nread + n
	    nreq = nreq - n
	    if (nreq .gt. 0) then
		call f_delete_blockette (hdr, -1)
		n = f_read_ms (hdr, data(nread+1), nreq, fp)
		if (n .gt. 0) then
		    if (hdr1.station_id .ne. hdr.station_id .or.
     1			hdr1.network_id .ne. hdr.network_id .or.
     2			hdr1.channel_id .ne. hdr.channel_id) then
			write (*,*) "Data block for wrong channel"
			goto 100
		    endif
		    slew = tslew(hdr1.begtime, hdr.begtime, nread,
     1			hdr.sample_rate)
		    if (slew .gt. thresh) then
			write (*,120) nread, slew, thresh
			goto 100
		    endif
		endif
	    else
		n = 0
	    endif
	end do


c we are finished reading data. check for error reading data.
100	if (n .lt. -1) then
		write(*,*) "Error reading file ", file
		i = ifclose (fp)
		call exit(1)
	endif
	
c output data points.
c convert times to printable forms.
c as an example, convert times to strings and to ext_time.
	call f_int_to_ext(hdr1.begtime, et1)
	call f_time_to_str(hdr1.begtime, date_fmt, s1)
c compute ending time (eg expected time of next data point).
	call f_time_interval (nread, hdr.sample_rate, seconds, usecs)
	call f_add_time(hdr1.begtime, seconds, usecs, endtime)
	call f_int_to_ext(endtime, et2)
	call f_time_to_str(endtime, date_fmt, s2)
c output header and data
	islew = slew
c	write (*,*) "station ", 
c     1		hdr1.station_id(1:lnblnk(hdr1.station_id)),
c     2		" channel ",
c     3		hdr1.channel_id(1:lnblnk(hdr1.channel_id)),
c     4		" time ", s1(1:lnblnk(s1)),
c     5		" points ", nread, " slew ", islew
c	do i = 1, nread
c		write (*,*) data(i)
c	end do
	call f_delete_blockette (hdr, -1)
	call f_delete_blockette (hdr1, -1)
	i = ifclose (fp)
120	format("Time error after ", i, " points -- slew ",f6.0, " gt", f4.0)
c
c ...... convert to local parameters
c
	lnb=lnblnk(hdr1.station_id)
	if(lnb.eq.3) then
	  stn=hdr1.station_id(1:lnblnk(hdr1.station_id))//' '
	else
	  stn=hdr1.station_id(1:lnblnk(hdr1.station_id))
	endif
	str=hdr1.channel_id(1:lnblnk(hdr1.channel_id))
	net=hdr1.network_id(1:lnblnk(hdr1.network_id))
	yr=et1.year
	doy=et1.doy
	hr=et1.hour
	mn=et1.minute
	sc=et1.second
	us=et1.usec
        dt=1.d0/dble(hdr1.sample_rate)
	np=nread
	do i = 1, nread
	  x(i)=dble(data(i))
	end do
c
	return
	end

	double precision function tslew (begtime, newtime, npts, rate)
c
c function to compute the total time slew from the beginning of data
c
	implicit none
 	include "/usr/local/include/qlib2.inc"
	record /INT_TIME/ begtime, newtime
	integer npts, rate
	double precision f_tdiff
	record /INT_TIME/ exptime
	integer seconds, usecs
c
	call f_time_interval (npts, rate, seconds, usecs)
	call f_add_time(begtime, seconds, usecs, exptime)
	tslew = f_tdiff(newtime, exptime)
c
	return
	end

	subroutine mswrite(file,stn,net,str,yr,doy,hr,mn,sc,us,
     1    dt,tshift,np,x)
c
c  Sample fortran program to read and uncompress data from 
c  a single channel MiniSEED data file.
c
	implicit none
 	include "/usr/local/include/qlib2.inc"

	integer BLKSIZE, MAXBLKS
	parameter (BLKSIZE = 4096, MAXBLKS = 100)
	character mseed*(BLKSIZE*MAXBLKS)
	integer MAXPTS
	parameter (MAXPTS = 180060)
	double precision THRESHOLD
	parameter (THRESHOLD = .5)

	record /DATA_HDR/ hdr
	record /INT_TIME/ it
	record /EXT_TIME/ et
	integer data(MAXPTS)
	integer fp, remaining, next, n, i
	integer nsamples, nrecords, nwrite, ndata
	integer seconds, usecs
	integer my_wordorder
c
c ...... declaration of parameters in subroutine call list
c
	character*4 stn
	character*3 str
	character*2 net
	integer*4 yr, doy, hr, mn, sc, us, np
	real*8 dt, tsms, tshift, x(*)
	character*80 file

c external functions and subroutines
	integer iargc
	external getarg, iargc
	integer lnblnk
	external lnblnk 
c functions in qlib2
	integer f_get_my_wordorder, f_mdy_to_doy
	external f_get_my_wordorder, f_mdy_to_doy
	external f_set_hdr_wordorder, f_set_data_wordorder
 	external f_ext_to_int, f_init_data_hdr, f_dy_to_mdy
	integer f_delete_blockette
	external f_delete_blockette
c functions in library fio.
	integer ifopen, ifclose, ifwrite
	external ifopen, ifclose, ifwrite

	
c open output miniseed file.
	fp = ifopen (file, "w")
	if (fp .eq. 0) then
		write(*,*) "unable to open file", file
		call exit(1)
	endif

c set number of requested data points.
c NOTE -- buffer must be large enough to handle this many points.
	nsamples = np
	do i = 1, nsamples
	    data(i) = x(i)
	end do

c Initialize the desired wordorder for the data.
	my_wordorder = f_get_my_wordorder()
	call f_set_hdr_wordorder(my_wordorder)
	call f_set_data_wordorder(my_wordorder)
c
c Compute the time of the first sample.
c
	et.year = yr
	et.doy = doy
	et.hour = hr
	et.minute = mn
	et.second = sc
	et.usec = us
	call f_dy_to_mdy (et.doy, et.year, et.month, et.day)
	call f_ext_to_int (et, it)
c
c ...... add time shift due the fir2caus transformation
c
	tsms=1000000.d0*tshift
	call f_add_dtime(it, tsms, it)
c
c Set up all of the required data in the header.
c
	call f_init_data_hdr (hdr)
	hdr.station_id = stn
	hdr.channel_id = str
	hdr.network_id = net
	hdr.sample_rate = idnint(1.d0/dt)
	hdr.data_type = STEIM1
	hdr.blksize = 4096
c
	hdr.hdrtime = it
	hdr.begtime = hdr.hdrtime
	hdr.seq_no = 1
	remaining = nsamples
c
c Loop over the data, packing it into MiniSEED and writing the 
c MiniSEED records to a file.  Continue until all data is processed.
c You REALLY want the mseed buffer large enough to hold your entire
c data array, since otherwise the MiniSEED compressor restarts
c every time you call it.
c
	do while (remaining .gt. 0)
	    next = nsamples - remaining + 1
	    hdr.num_samples = remaining
c
c Create the MiniSEED data.  If the output buffer is not large enough
c hold the packed MiniSEED for all of the input data, only a portion
c of the data will be packed.  The function will return the number
c of data points packed into the MiniSEED buffer.
c
	    ndata = f_ms_pack_data (hdr, hdr.num_samples, data(next), 
     1				nrecords, mseed, BLKSIZE * MAXBLKS)
c
c Write the MiniSEED data to the file.
c
	    nwrite = ifwrite (mseed, hdr.blksize * nrecords, 1, fp)
	    if (nwrite .ne. 1) then
		write (*,'(A A)') 'Error writing MiniSEED file', file
		call exit (1)
	    endif
	
c
c Update the hdr for the next chunk of data
c
	    call f_time_interval (ndata, hdr.sample_rate, seconds, usecs)
	    call f_add_time(hdr.hdrtime, seconds, usecs, hdr.hdrtime)
	    hdr.begtime = hdr.hdrtime
	    n = f_delete_blockette (hdr, -1)
	    remaining = remaining - ndata
	    if (remaining .gt. 0) then
		write (*,'(A,I,A)' ) 'Warning - mseed buffer too small',
     1				   remaining, ' points process next pass.'
	    endif
	end do
c
c CLose the file.
c
	i = ifclose (fp)
c
	return
	end
