	subroutine mswrite(file)
c
c  Sample fortran program to read and uncompress data from 
c  a single channel MiniSEED data file.
c
	implicit none
 	include "/usr/local/include/qlib2.inc"

	integer BLKSIZE, MAXBLKS
	parameter (BLKSIZE = 4096, MAXBLKS = 100)
	character mseed*(BLKSIZE*MAXBLKS)
	integer MAXPTS
	parameter (MAXPTS = 3500000)
	double precision THRESHOLD
	parameter (THRESHOLD = .5)

	record /DATA_HDR/ hdr
	record /INT_TIME/ it
	record /EXT_TIME/ et
	integer data(MAXPTS)
	integer fp, remaining, next, n, i
	integer nsamples, nrecords, nwrite, ndata
	integer seconds, usecs, nargs
	integer my_wordorder
	character*80 file

c external functions and subroutines
	integer iargc
	external getarg, iargc
	integer lnblnk
	external lnblnk 
c functions in qlib2
	integer f_get_my_wordorder, f_mdy_to_doy
	external f_get_my_wordorder, f_mdy_to_doy
	external f_set_hdr_wordorder, f_set_data_wordorder
 	external f_ext_to_int, f_init_data_hdr
	integer f_delete_blockette
	external f_delete_blockette
c functions in library fio.
	integer ifopen, ifclose, ifwrite
	external ifopen, ifclose, ifwrite

	
c read filename from command line argument and open input miniseed file.
c	nargs = iargc()
c	if (nargs .lt. 1) then
c	    write (*,*) "Missing filename"
c	    call exit(1)
c	endif
c	call getarg (1, file)		
c	
	fp = ifopen (file, "w")
	if (fp .eq. 0) then
		write(*,*) "unable to open file", file
		call exit(1)
	endif

c set number of requested data points.
c NOTE -- buffer must be large enough to handle this many points.
c	nsamples = 50000
c	do i = 1, nsamples
c	    data(i) = i
c	end do

c Initialize the desired wordorder for the data.
	my_wordorder = f_get_my_wordorder()
	call f_set_hdr_wordorder(my_wordorder)
	call f_set_data_wordorder(my_wordorder)
c
c Compute the time of the first sample.
c
c	et.year = 1996
c	et.month = 12
c	et.day = 25
c	et.hour = 8
c	et.minute = 25
c	et.second = 30.
c	et.usec = 500000
c	et.doy = f_mdy_to_doy (et.month, et.day, et.year)
c	call f_ext_to_int (et, it)
c
c Set up all of the required data in the header.
c
c	call f_init_data_hdr (hdr)
c	hdr.station_id = 'TEST'
c	hdr.channel_id = 'CHN'
c	hdr.network_id = 'XX'
c	hdr.sample_rate = 20
c	hdr.data_type = STEIM1
c	hdr.blksize = 4096
c
c	hdr.hdrtime = it
c	hdr.begtime = hdr.hdrtime
c	hdr.seq_no = 1
c	remaining = nsamples
c
c Loop over the data, packing it into MiniSEED and writing the 
c MiniSEED records to a file.  Continue until all data is processed.
c You REALLY want the mseed buffer large enough to hold your entire
c data array, since otherwise the MiniSEED compressor restarts
c every time you call it.
c
	do while (remaining .gt. 0)
	    next = nsamples - remaining + 1
	    hdr.num_samples = remaining
c
c Create the MiniSEED data.  If the output buffer is not large enough
c hold the packed MiniSEED for all of the input data, only a portion
c of the data will be packed.  The function will return the number
c of data points packed into the MiniSEED buffer.
c
	    ndata = f_ms_pack_data (hdr, hdr.num_samples, data(next), 
     1				nrecords, mseed, BLKSIZE * MAXBLKS)
c
c Write the MiniSEED data to the file.
c
	    nwrite = ifwrite (mseed, hdr.blksize * nrecords, 1, fp)
	    if (nwrite .ne. 1) then
		write (*,'(A A)') 'Error writing MiniSEED file', file
		call exit (1)
	    endif
	
c
c Update the hdr for the next chunk of data
c
	    call f_time_interval (ndata, hdr.sample_rate, seconds, usecs)
	    call f_add_time(hdr.hdrtime, seconds, usecs, hdr.hdrtime)
	    hdr.begtime = hdr.hdrtime
	    n = f_delete_blockette (hdr, -1)
	    remaining = remaining - ndata
	    if (remaining .gt. 0) then
		write (*,'(A,I,A)' ) 'Warning - mseed buffer too small',
     1				   remaining, ' points process next pass.'
	    endif
	end do
c
c CLose the file.
c
	i = ifclose (fp)
c
	return
	end
