	program firfft
c
c ...... Fourier Spectrum of FIR Filter
c
	implicit none
	integer*4 i, j, k, nc, np, lnblnk, lnb, narg, iargc, 
     1    system, isign, jj, deci, nf
	real*8 data(2048), sum, sps, gfreq, gain, onfreq, theta
	real*8 fc(1024), amp(2048), ang(2048), freq(2048), phase
	character*80 infile, clarg, bline
	character*4 parity
c
	narg=iargc()
	if(narg.ne.5) then
	  write(*,*) 'fir_fp - FIR filter gain'
	  write(*,*) ' ' 
	  write(*,*) 'Usage: file parity deci sps gfreq'
	  write(*,*) '   file    - FIR coefficient file name'
	  write(*,*) '   parity  - Parity of filter'
	  write(*,*) 
     1      '             (must be either "Even" or "None")'
	  write(*,*) '   deci    - Filter decimation factor'
	  write(*,*) '   sps     - Input sample rate'
	  write(*,*) 
     1      '   gfreq   - Frequency at which gain is calculated'
	  write(*,*) ' '
	  write(*,*) 
     1      'Output: file parity deci sps ofn dc_gain g_freq ',
     2      'gain phase'
	  write(*,*) '   file    - FIR coefficient file name'
	  write(*,*) '   parity  - Parity of filter'
	  write(*,*) '   deci    - Filter decimation factor'
	  write(*,*) '   sps     - Input sample rate'
	  write(*,*) '   ofn     - Output Nyquist frequency'
	  write(*,*) '   dc_gain - Gain at zero frequency'
	  write(*,*) '   g_freq  - Frequency'
	  write(*,*) '   gain    - Gain at g_freq'
	  write(*,*) '   phase   - Phase at g_freq'
	  call exit(1)
	endif
c
	i=1
	call getarg(i,clarg)
	lnb=lnblnk(clarg)
	read(clarg(1:lnb),'(a)') infile
	i=2
	call getarg(i,clarg)
	lnb=lnblnk(clarg)
	read(clarg(1:lnb),'(a)') parity
	i=3
	call getarg(i,clarg)
	lnb=lnblnk(clarg)
	read(clarg(1:lnb),'(i)') deci
	i=4
	call getarg(i,clarg)
	lnb=lnblnk(clarg)
	read(clarg(1:lnb),'(f)') sps
	i=5
	call getarg(i,clarg)
	lnb=lnblnk(clarg)
	read(clarg(1:lnb),'(f)') gfreq
c
	lnb=lnblnk(infile)
	open(10,file=infile(1:lnb),status='old')
	bline=
     1    "grep fir_coeff "//infile(1:lnb)//
     2    " | awk '{ print $2 }' > foo_1"
	lnb=lnblnk(bline)
	i=system(bline(1:lnb))
	open(10,file='foo_1',status='old')
c
	i=0
    1   i=i+1
	read(10,'(f)',end=2) fc(i)
	go to 1
    2   continue
	nc=i-1
	close(10)
c
	sum=0.d0
	j=0
	do 3 i=1,nc
	  j=j+1
	  data(j)=fc(i)
	  sum=sum+fc(i)
	  j=j+1
	  data(j)=0.d0
c	  write(*,*) 'fc(',i,') = ',fc(i)
    3   continue
	if(parity.eq.'Even') then
	  do 35 i=nc,1,-1
	    j=j+1
	    data(j)=fc(i)
	    sum=sum+fc(i)
c	    write(*,*) 'fc(',2*nc+1-i,') = ',fc(i)
	    j=j+1
	    data(j)=0.d0
   35     continue
	endif
	np=1024
	jj=j
	do 4 i=jj+1,np
	  j=j+1
	  data(j)=0.d0
	  j=j+1
	  data(j)=0.d0
    4   continue
c
	isign=-1
	call four1(data,np,isign)
c
	j=0
	k=1
	freq(k)=0.d0
	amp(k)=sum
c	write(*,'(f15.5,f20.12)') freq,sum
	do 5 i=3,np+2,2
	  k=k+1
	  j=j+1
	  amp(k)=dsqrt(data(i)*data(i)+data(i+1)*data(i+1))
	  freq(k)=dble(j)/(dble(np)/sps)
	  if((data(i+1).eq.0.d0).and.(data(i).eq.0.d0)) then
	    ang(k)=0.d0
	  else
	    ang(k)=datan2d(data(i+1),data(i))
	    if(parity.eq.'Even') then
	      theta=360.d0*dble(2*nc-1)*freq(k)/(2.d0*sps)
	      theta=dmod(theta,360.d0)
	      if(theta.gt.180.d0) theta=theta-360.d0
	    else
	      theta=360.d0*dble(nc-1)*freq(k)/sps
	      theta=dmod(theta,360.d0)
	      if(theta.gt.180.d0) theta=theta-360.d0
	    endif
	    ang(k)=ang(k)+theta
	    if(ang(k).lt.-180.d0) ang(k)=ang(k)+360.d0
	    if(ang(k).gt.+180.d0) ang(k)=ang(k)-360.d0!
	  endif
	  freq(k)=dble(j)/(dble(np)/sps)
c	  write(*,'(f15.5,f20.12,f15.5)') freq(j), amp(j), ang(j)
    5   continue
	nf=k
c
	call gfcalc(nf,freq,amp,ang,gfreq,gain,phase)
	onfreq=sps/(2.d0*dble(deci))
	write(*,'(a," ",a4," ",i2," ",f9.3," ",f10.5," ",f12.10," ",
     1    f9.5," ",f12.10," ",f10.5)')
     2    infile(1:lnblnk(infile)),parity(1:4),deci,sps,onfreq,sum,
     3    gfreq,gain,phase
c
	stop
	end

	subroutine gfcalc(nf,freq,amp,ang,gfreq,gain,phase)
c
c ...... determine gain at target frequency
c
	implicit none
	integer*4 i, nf
	real*8 freq(*), amp(*), ang(*)
	real*8 gfreq, gain, dadf, phase, dpdf
c
	i=0
    1   i=i+1
	if(i+1.eq.nf) then
	  write(*,*) 'frequency not valid'
	  call exit(1)
	endif
	if(freq(i).lt.gfreq) go to 1
	dadf=(amp(i)-amp(i-1))/(freq(i)-freq(i-1))
	gain=amp(i-1)+dadf*(gfreq-freq(i-1))
	dpdf=(ang(i)-ang(i-1))/(freq(i)-freq(i-1))
	phase=ang(i-1)+dpdf*(gfreq-freq(i-1))
c
	return
	end

	subroutine four1(data,nn,isign)
c
c ...... discrete fourier transform
c
	real*8 wr, wi, wpr, wpi, wtemp, theta
	real*8 data(2048), tempr, tempi
c
	n=2*nn
	j=1
	do 11 i=1,n,2
	  if(j.gt.i) then
	    tempr=data(j)
	    tempi=data(j+1)
	    data(j)=data(i)
	    data(j+1)=data(i+1)
	    data(i)=tempr
	    data(i+1)=tempi
	  endif
	  m=n/2
    1     if((m.ge.2).and.(j.gt.m)) then
	    j=j-m
	    m=m/2
	    go to 1
	  endif
	  j=j+m
   11   continue
	mmax=2
    2   if(n.gt.mmax) then
	  istep=2*mmax
	  theta=6.28318530717959d0/(isign*mmax)
	  wpr=-2.d0*dsin(0.5d0*theta)**2
	  wpi=dsin(theta)
	  wr=1.d0
	  wi=0.d0
	  do 13 m=1,mmax,2
	    do 12 i=m,n,istep
	      j=i+mmax
	      tempr=wr*data(j)-wi*data(j+1)
	      tempi=wr*data(j+1)+wi*data(j)
	      data(j)=data(i)-tempr
	      data(j+1)=data(i+1)-tempi
	      data(i)=data(i)+tempr
	      data(i+1)=data(i+1)+tempi
   12       continue
	    wtemp=wr
	    wr=wr*wpr-wi*wpi+wr
	    wi=wi*wpr+wtemp*wpi+wi
   13     continue
	  mmax=istep
	  go to 2
	endif
c
	return
	end
